
using System.Diagnostics.CodeAnalysis;
using System.Threading;

namespace Dashboard.CommandMachine
{
    public interface ICommandListSerializable { }

    /// <summary>
    /// Interface for objects that can be serialized into the Dashboard command stream.
    /// </summary>
    public interface ICommandListSerializable<T> : ICommandListSerializable
    {
        /// <summary>
        /// Seralize object.
        /// </summary>
        /// <param name="list">The object to serialize into.</param>
        void Serialize(CommandList list);

        /// <summary>
        /// Deserialize object.
        /// </summary>
        /// <param name="queue">The command queue to deserialize from.</param>
        void Deserialize(CommandQueue queue);
    }

    /// <summary>
    /// Base interface for all Command List serializers.
    /// </summary>
    public interface ICommandListSerializer { }

    public interface ICommandListSerializer<T> : ICommandListSerializer
    {
        /// <summary>
        /// Serialize an object into the command list.
        /// </summary>
        /// <param name="value">The object to serialize.</param>
        /// <param name="list">The command list to serialize into.</param>
        void Serialize(T value, CommandList list);

        /// <summary>
        /// Deserialize an object from the command queue.
        /// </summary>
        /// <param name="queue">The command queue.</param>
        /// <returns>The object deserialized from the command queue.</returns>
        [return: NotNull]
        T Deserialize(CommandQueue queue);
    }

    /// <summary>
    /// Class for automatic serialization of <see cref="ICommandListSerializable"/> objects.
    /// </summary>
    /// <typeparam name="T">The object type to convert.</typeparam>
    internal class CommandListSerializableSerializer<T> : ICommandListSerializer<T>
        where T : ICommandListSerializable<T>, new()
    {
        public T Deserialize(CommandQueue queue)
        {   
            T value = new T();
            value.Deserialize(queue);
            return value;
        }

        public void Serialize(T value, CommandList list)
        {
            value.Serialize(list);
        }
    }
}